#!/usr/bin/python
#
# Copyright INRA-URGI 2009-2010
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
#
from commons.core.LoggerFactory import LoggerFactory
from commons.core.utils.RepetOptionParser import RepetOptionParser

LOG_DEPTH = "smart"

class AddUtr(object):

	def __init__(self, verbosity = 0):
		self.verbosity = verbosity
		self.log       = LoggerFactory.createLogger("%s.%s" % (LOG_DEPTH, self.__class__.__name__), self.verbosity)
		self.cds       = {}

	def setInputFileName(self, fileName):
		self.inputFileName = fileName

	def setOutputFileName(self, fileName):
		self.outputFileName = fileName

	def parseTags(self, tags):
		parsedTags = {}
		for tag in tags.split(";"):
			key, value = tag.split("=")
			parsedTags[key] = value
		return parsedTags

	def createUtr(self, transcript, id, minPos, maxPos, utrEnd):
		if minPos > maxPos:
			return ""
		utr    = transcript
		utr[2] = utrEnd
		utr[8] = "ID=%s_%s;Name=%s_%s;Parent=%s" % (id, utrEnd, id, utrEnd, id)
		utr[3] = "%d" % (minPos)
		utr[4] = "%d" % (maxPos)
		return "%s\n" % ("\t".join(utr))
		
	def parseInput(self):
		inputFile = open(self.inputFileName)
		for line in inputFile:
			splittedLine = line.strip().split(None, 8)
			if splittedLine[2] == "CDS":
				parsedTags   = self.parseTags(splittedLine[8])
				position     = (int(splittedLine[3]), int(splittedLine[4]))
				parent       = parsedTags["Parent"]
				if parent not in self.cds:
					self.cds[parent] = [position]
				else:
					self.cds[parent].append(position)
		inputFile.close()

	def writeOutput(self):
		outputFile = open(self.outputFileName, "w")
		inputFile  = open(self.inputFileName)
		for line in inputFile:
			outputFile.write(line)
			splittedLine = line.strip().split(None, 8)
			if splittedLine[2] == "transcript":
				parsedTags = self.parseTags(splittedLine[8])
				id         = parsedTags["ID"]
				if id in self.cds:
					minPos      = int(splittedLine[3])
					maxPos      = int(splittedLine[4])
					sortedExons = sorted(self.cds[id], key=lambda t: t[0])
					minUtr      = (minPos, sortedExons[0][0]-1)
					maxUtr      = (sortedExons[-1][1]+1, maxPos)
					if splittedLine[6] == "+":
						five  = self.createUtr(splittedLine, id, minUtr[0], minUtr[1], "five_prime_utr")
						three = self.createUtr(splittedLine, id, maxUtr[0], maxUtr[1], "three_prime_utr")
					else:
						three = self.createUtr(splittedLine, id, minUtr[0], minUtr[1], "three_prime_utr")
						five  = self.createUtr(splittedLine, id, maxUtr[0], maxUtr[1], "five_prime_utr")
					outputFile.write(five)
					outputFile.write(three)
		inputFile.close()
		outputFile.close()

	def run(self):
		LoggerFactory.setLevel(self.log, self.verbosity)
		self.log.info("START Add UTR")
		self.parseInput()
		self.writeOutput()
		self.log.info("END Add UTR")


if __name__ == "__main__":
	description = "Usage: AddUtr.py [options]\n\nAdd UTR v1.0.1: Add UTR lines in a GFF file. [Category: Personal]\n"
	epilog = ""
	parser = RepetOptionParser(description = description, epilog = epilog)
	parser.add_option("-i", "--input",     dest="inputFileName",   action="store",      default=None,      type="string", help="input file [compulsory] [format: transcript file in GFF format]")
	parser.add_option("-o", "--output",    dest="outputFileName",  action="store",      default=None,      type="string", help="output file [format: output transcript file in GFF format]")
	parser.add_option("-v", "--verbosity", dest="verbosity",       action="store",      default=1,         type="int",    help="trace level [format: int]")
	options = parser.parse_args()[0]
	iAddUtr = AddUtr(options.verbosity)
	iAddUtr.setInputFileName(options.inputFileName)
	iAddUtr.setOutputFileName(options.outputFileName)
	iAddUtr.run()

