#! /usr/bin/env python
#
# Copyright INRA-URGI 2009-2011
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software. You can use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
# 
# As a counterpart to the access to the source code and rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty and the software's author, the holder of the
# economic rights, and the successive licensors have only limited
# liability.
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading, using, modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean that it is complicated to manipulate, and that also
# therefore means that it is reserved for developers and experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and, more generally, to use and operate it in the
# same conditions as regards security.
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
#
import os, random
from optparse import OptionParser, OptionGroup
from SMART.Java.Python.structure.TranscriptContainer import TranscriptContainer
from SMART.Java.Python.misc.Progress import Progress
from SMART.Java.Python.misc.UnlimitedProgress import UnlimitedProgress
from commons.core.writer.Gff3Writer import Gff3Writer

NB_FILES = 2
NB_LINES = 4

class UninterleaveFastq(object):

	def __init__(self, verbosity = 0):
		self.verbosity = verbosity
		self.writers   = [None, None]

	def setInputFileName(self, fileName):
		self.parsers = open(fileName)
	
	def setOutputFileName(self, fileName, index):
		self.writers[index] = open(fileName, "w")

	def getLines(self):
		lines = ["", ""]
		for i in range(NB_FILES):
			for j in range(NB_LINES):
				line = self.parsers.readline()
				if not line:
					return False
				lines[i] += line
		return lines

	def run(self):
		lines    = self.getLines()
		progress = UnlimitedProgress(100000, "Writing file", self.verbosity)
		while lines:
			for i in range(NB_FILES):
				self.writers[i].write(lines[i])
			lines = self.getLines()
			progress.inc()
		progress.done()

if __name__ == "__main__":
	
	# parse command line
	description = "Uninterleave Fastq v1.0.1: Uninterleave Fastq files. [Category: Personal]"

	parser = OptionParser(description = description)
	parser.add_option("-i", "--input",	   dest="inputFileName",   action="store", type="string", help="left read file [compulsory] [format: file in FASTQ format]")
	parser.add_option("-1", "--output1",   dest="outputFileName1", action="store", type="string", help="output file 1 [format: output file in FASTQ format]")
	parser.add_option("-2", "--output2",   dest="outputFileName2", action="store", type="string", help="output file 2 [format: output file in FASTQ format]")
	parser.add_option("-v", "--verbosity", dest="verbosity",	   action="store", type="int",    help="trace level [default: 1] [format: int]")
	(options, args) = parser.parse_args()

	uninterleave = UninterleaveFastq(options.verbosity)
	uninterleave.setInputFileName(options.inputFileName)
	uninterleave.setOutputFileName(options.outputFileName1, 0)
	uninterleave.setOutputFileName(options.outputFileName2, 1)
	uninterleave.run()

